#pragma once



#include <math.h>
#include <vector>
#include "windows.h"
#include <d2d1.h>
#include "Colors.h"

// Common Constants

#define PI_F 3.1415927f

template <typename T>
class _Vec2
{
public:
	inline			_Vec2() {}
	inline			_Vec2(T x, T y)
		:
		x(x),
		y(y)
	{}
	inline			_Vec2(const _Vec2& vect)
		:
		_Vec2(vect.x, vect.y)
	{}
	template <typename T2>
	inline			operator _Vec2< T2 >() const
	{
		return{ (T2)x,(T2)y };
	}
	inline T		LenSq() const
	{
		return *this * *this;
	}
	inline T		Len() const
	{
		return sqrt(LenSq());
	}
	inline _Vec2&	Normalize()
	{
		const T length = Len();
		x /= length;
		y /= length;
		return *this;
	}
	inline _Vec2&	CCW90()
	{
		T temp = y;
		y = -x;
		x = temp;
		return *this;
	}
	inline _Vec2&	CW90()
	{
		T temp = y;
		y = x;
		x = -temp;
		return *this;
	}
	inline _Vec2&	Swap(_Vec2& vect)
	{
		const _Vec2 temp = vect;
		vect = *this;
		*this = temp;
		return *this;
	}
	inline _Vec2	operator-() const
	{
		return _Vec2(-x, -y);
	}
	inline _Vec2&	operator=(const _Vec2 &rhs)
	{
		this->x = rhs.x;
		this->y = rhs.y;
		return *this;
	}
	inline _Vec2&	operator+=(const _Vec2 &rhs)
	{
		this->x += rhs.x;
		this->y += rhs.y;
		return *this;
	}
	inline _Vec2&	operator-=(const _Vec2 &rhs)
	{
		this->x -= rhs.x;
		this->y -= rhs.y;
		return *this;
	}
	inline T		operator*(const _Vec2 &rhs) const
	{
		return this->x * rhs.x + this->y * rhs.y;
	}
	inline _Vec2	operator+(const _Vec2 &rhs) const
	{
		return _Vec2(*this) += rhs;
	}
	inline _Vec2	operator-(const _Vec2 &rhs) const
	{
		return _Vec2(*this) -= rhs;
	}
	inline _Vec2&	operator*=(const T &rhs)
	{
		this->x *= rhs;
		this->y *= rhs;
		return *this;
	}
	inline _Vec2	operator*(const T &rhs) const
	{
		return _Vec2(*this) *= rhs;
	}
	inline _Vec2&	operator/=(const T &rhs)
	{
		this->x /= rhs;
		this->y /= rhs;
		return *this;
	}
	inline _Vec2	operator/(const T &rhs) const
	{
		return _Vec2(*this) /= rhs;
	}
	inline T		CrossWith(const _Vec2 &rhs) const
	{
		return x * rhs.y - y * rhs.x;
	}
	inline _Vec2	ClosestPtOnLine(const _Vec2 &lnpt1, const _Vec2 &lnpt2) const
	{
		_Vec2 p1 = lnpt1;
		_Vec2 p2 = lnpt2;
		_Vec2 pointOnLine;
		if (abs(p1.x - p2.x) > 0.0f)
		{
			if (p1.x > p2.x)
			{
				p1.Swap(p2);
			}
			const T m = (p2.y - p1.y) / (p2.x - p1.x);
			const T b = p1.y - m * p1.x;

			if (abs(m) > 0.0f)
			{
				T n = -1.0f / m;
				T c = this->y - this->x * n;
				pointOnLine.x = (b - c) / (n - m);
				pointOnLine.y = m * pointOnLine.x + b;
			}
			else
			{
				pointOnLine.x = this->x;
				pointOnLine.y = p1.y;
			}
		}
		else
		{
			pointOnLine.x = p1.x;
			pointOnLine.y = this->y;
		}
		return pointOnLine;
	}
	inline bool		IsInsideRect(const _Vec2 &p1, const _Vec2 & p2) const
	{
		const T medianX = (p1.x + p2.x) / 2.0f;
		const T medianY = (p1.y + p2.y) / 2.0f;
		return abs(medianX - x) <= abs(medianX - p1.x) &&
			abs(medianY - y) <= abs(medianY - p1.y);
	}
	inline bool		operator==(const _Vec2 &rhs) const
	{
		return x == rhs.x && y == rhs.y;
	}
	inline bool		operator!=(const _Vec2 &rhs) const
	{
		return !(*this == rhs);
	}
	inline _Vec2	MidpointWith(const _Vec2& p2) const
	{
		return _Vec2((x + p2.x) / 2.0f, (y + p2.y) / 2.0f);
	}
	inline _Vec2    Rotation(float angle)
	{
		_Vec2<T> result;
		float c = cosf(angle);
		float s = sinf(angle);
		result.x = x * c - y * s;
		result.y = x * s + y * c;
		return result;
	}

public:
	union
	{
		struct
		{
			T x;
			T y;
		};
		struct
		{
			T cols;
			T rows;
		};
		struct
		{
			T width;
			T height;
		};
	};
};


template < typename T >
inline std::vector < _Vec2<T>> CalculateIntersectionPoints(_Vec2<T> q, _Vec2<T> p1, _Vec2<T> p2, T r)
{
	std::vector<_Vec2<T>> points;
	_Vec2<T> d = p2 - p1;

	float dr2 = d.LenSq();
	float D = (p1 - q).CrossWith(p2 - q);
	float disc = sq(r) * dr2 - sq(D);
	if (disc >= (T)0.0)
	{
		float sqrtDisc = sqrt(disc);
		float lhsx = D * d.y;
		float rhsx = sgn(d.y)* d.x * sqrtDisc;
		float lhsy = -D * d.x;
		float rhsy = abs(d.y) * sqrtDisc;
		points.push_back(_Vec2 < T > {(lhsx + rhsx) / dr2, (lhsy + rhsy) / dr2}+q);
		if (disc > (T)0.0)
			points.push_back(_Vec2 < T > {(lhsx - rhsx) / dr2, (lhsy - rhsy) / dr2}+q);
	}
	return points;
};


template <class T> 
class Vector4
{
public:
	Vector4() {}
	
	union
	{
		struct
		{
			T x;
			T y;
			T z;
			T w;
		};
		struct
		{
			T r;
			T g;
			T b;
			T a;
		};
		struct
		{
			T left;
			T top;
			T right;
			T bottom;
		};
		struct
		{
			T X;
			T Y;
			T width;
			T height;
		};
	};
	// left,top,right,bottom
	bool Overlaps(Vector4& other)
	{
		return (this->left < other.right && this->right > other.left &&
			this->top < other.bottom && this->bottom > other.top);
	}
	T& operator[](unsigned int index)
	{
		return static_cast<T*>(this)[index];
	}
	D2D1_RECT_F ToD2DRect()
	{
		return D2D1::RectF((float)left, (float)top, (float)right, (float)bottom);
	}
	template <typename T2>
	bool PointIn(_Vec2<T> T2)
	{
		return (T2.x > (T)left && T2.x < (T)right && T2.y >(T)top && T2.y < (bottom));
	}
    Vector4(T _x , T _y , T _z, T _w ) : x(_x), y(_y), z(_z), w(_w) { }
};
typedef unsigned int uint;
typedef Vector4<float> Vec4f;
typedef Vector4<int> Vec4i;
typedef Vector4<float> Color4;
typedef _Vec2< float > Vec2f;
typedef _Vec2< int > Vec2i;
typedef _Vec2< double > Vec2D;
typedef _Vec2<float> float2;
typedef _Vec2<size_t>  uint2;
typedef _Vec2<int>  int2;

template <class T>
Vector4<T> operator-(Vector4<T> a, Vector4<T> b)
{
	return Vector4<T>(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
}

template <class T>
Vector4<T> operator-(Vector4<T> a)
{
	return Vector4<T>(-a.x, -a.y, -a.z, -a.w);
}



template <class T>
Vector4<T> operator+(Vector4<T> a, Vector4<T> b)
{
	return Vector4<T>(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
}






template <class T>
Vector4<T> operator*(Vector4<T> a, T s)
{
	return Vector4<T>(a.x * s, a.y * s, a.z * s, a.w * s);
}

template <class T>
Vector4<T> operator*(T s, Vector4<T> a)
{
	return a * s;
}

template <class T>
Vector4<T> operator*(Vector4<T> a, Vector4<T> b)
{
	return Vector4<T>(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
}

template <class T>
Vector4<T> operator/(Vector4<T> a, T s)
{
	return Vector4<T>(a.x / s, a.y / s, a.z / s, a.w / s);
}


// Get sign of number (+-)
// returns -1 or 1
template <typename T>
static int sgn(T val)
{
	return (T(0) < val) - (val < T(0));
}

enum class Game_State
{
	intro,
	paused,
	over,
	start,
	running,
	load,
	exit
};

struct IMAGEINFO
{
	std::vector<Color> data;
	int width;
	int height;
	int rows;
	int columns;
	std::string filename;

};





